// GLTemplateView.cpp : implementation of the CGLTemplateView class
//

#include "stdafx.h"
#include "GLTemplate.h"

#include "GLTemplateDoc.h"
#include "GLTemplateView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGLTemplateView

IMPLEMENT_DYNCREATE(CGLTemplateView, CView)

BEGIN_MESSAGE_MAP(CGLTemplateView, CView)
	//{{AFX_MSG_MAP(CGLTemplateView)
	ON_WM_ERASEBKGND()
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGLTemplateView construction/destruction

CGLTemplateView::CGLTemplateView() :
	m_hDC(0), m_hRC(0)
{
}

CGLTemplateView::~CGLTemplateView()
{
}

BOOL CGLTemplateView::PreCreateWindow(CREATESTRUCT& cs)
{
	cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CGLTemplateView drawing

void CGLTemplateView::OnDraw(CDC* pDC)
{
	CGLTemplateDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	wglMakeCurrent( m_hDC, m_hRC );

	// here you can begin drawing with GL commands:

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	// draw a triangle.

	glBegin( GL_TRIANGLES );

	glColor3f( 1.0f, 0.0f, 0.0f );
	glVertex3f( 0.0f, 0.0f, 0.0f );

	glColor3f( 0.0f, 1.0f, 0.0f );
	glVertex3f( 1.0f, 0.0f, 0.0f );

	glColor3f( 0.0f, 0.0f, 1.0f );
	glVertex3f( 0.0f, 1.0f, 0.0f );

	glEnd();

	// now flush the GL pipeline and swap buffers to reveal what you've just drawn.

	glFlush();
	SwapBuffers( m_hDC );
	wglMakeCurrent( NULL, NULL );
}


/////////////////////////////////////////////////////////////////////////////
// CGLTemplateView diagnostics

#ifdef _DEBUG
void CGLTemplateView::AssertValid() const
{
	CView::AssertValid();
}

void CGLTemplateView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CGLTemplateDoc* CGLTemplateView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CGLTemplateDoc)));
	return (CGLTemplateDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CGLTemplateView message handlers

BOOL CGLTemplateView::OnEraseBkgnd(CDC* pDC) 
{
	return TRUE;
}

int CGLTemplateView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	m_hDC = ::GetDC( m_hWnd );

	PIXELFORMATDESCRIPTOR pfd = {
		sizeof(PIXELFORMATDESCRIPTOR),
		1,
		PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER,
		PFD_TYPE_RGBA,
		24,
		0, 0, 0, 0, 0, 0, 
		0, 0, 0,
		0, 0, 0, 0,
		32,
		0, 0,
		PFD_MAIN_PLANE,
		0,
		0, 0, 0 };
	int pixelformat;

	pixelformat = ChoosePixelFormat( m_hDC, &pfd );
	if ( pixelformat == 0 )
	{
		MessageBox( "ChoosePixelFormat failed.\n" );
		return -1;
	}

	if ( FALSE == SetPixelFormat( m_hDC, pixelformat, &pfd ) )
	{
		MessageBox( "SetPixelFormat failed.\n" );
		return -1;
	}

	m_hRC = wglCreateContext( m_hDC );
	wglMakeCurrent( m_hDC, m_hRC );
	SetupGLParameters();
	wglMakeCurrent( NULL, NULL );
	

	return 0;
}

void CGLTemplateView::SetupGLParameters( void )
{
	// this function is called once when the GL window is created.  you can use
	// it to initialize some GL state.

	glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
	glClearDepth( 1.0f );
	glEnable( GL_DEPTH_TEST );
}

void CGLTemplateView::OnDestroy() 
{
	CView::OnDestroy();
	
	wglMakeCurrent( NULL, NULL );
	wglDeleteContext( m_hRC );
	::ReleaseDC( m_hWnd, m_hDC );
}

void CGLTemplateView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	if ( 0 >= cx || 0 >= cy )
		return;

	wglMakeCurrent( m_hDC, m_hRC );
	SetupProjection( cx, cy );
	wglMakeCurrent( NULL, NULL );
}


void CGLTemplateView::SetupProjection( int cx, int cy )
{
	// this function is called when the window is created and whenever it is resized.
	// it sets the projection matrix to correspond to the window's aspect ratio.

	GLdouble aspect_ratio;

	glViewport( 0, 0, cx, cy );

	aspect_ratio = (GLdouble)cx/ (GLdouble)cy;
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 40.0f, aspect_ratio, .1f, 20.0f );
	
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();

	// the default camera is placed at the origin, looking in the -Z direction, with 
	// "up" in the +Y direction.

	// back up and rotate the camera a little so we are looking at the origin.

	glTranslatef( 0.0f, 0.0f, -5.0f );
	glRotatef( 20.0f, 1.0f, 0.0f, 0.0f );
}
